==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webmail/server/inc/imap/folder.php <==
<?php

require_once('inc/cache/folder.php');
require_once('inc/imap/imap.php');

/**
  @class IMAPFolder
  @brief Class representing an IMAP folder.
  @author Martin Zlomek
  @date 09-02-2006
 */
class IMAPFolder extends CacheFolder {

	//Type of the folder == M for mail items 
	public $type;  
	
	/**
	  @brief Create a new IMAP folder object.
	  @param $account Parent account object of the folder.
	  @param $folderID ID of the folder.
	  @param $name Name of the folder.
	  @param $rights Rights to the folder.
	 */	 
	public function __construct(&$account, $folderID, $name, $rights, $sync)
	{
		parent::__construct($account, $folderID, $name, $rights, $sync);
		$this->type = 'M';
		$this->gw = false;
		$this->itemClassName = 'IMAPItem';
	}

	/**
	  @brief Create a new folder.
	  @param $account Parent account object of the folder.
	  @param $name Name of the folder.
	  @return IMAPFolder object.
	 */
	public static function create($account, $name)
	{
    /* Create the folder on the IMAP server. */
		$imap = IMAP::instance($account);
		$rights = self::decodeRights($imap->createMailbox($name, $account->accountID));

		/* Create the folder in the folder cache. */
		$folderID = parent::create($account, $name, $rights);

		return new IMAPFolder($account, $folderID, $name, $rights, false);
	}


	/**
	  @brief Delete the folder.
	 */
	public function delete()
	{
		$account = $this->account;
		
		
	
		//Get parent folder
    $oFolderParent = $this->getParent($account);

  	$imap = IMAP::instance($account);

    $folders = krsort($account->folders["main"]);

		/* Delete all sub-folders from the IMAP server. */
		foreach ($account->folders["main"] as $folder) {
			if (DEBUG) echo "Check for delete : ".$folder->name.'-'.$this->name.'/'."\r";
			
      if (strpos($folder->name, $this->name . '/') !== 0)
				continue;
				
			if (DEBUG) echo "Delete:".$folder->name.'-'.$this->name.'/'."\r";
			@$imap->deleteMailbox($folder->name);

			unset($account->folders['main'][$folder->name]);
		}   

		/* Delete the folder itself from the IMAP server. */
		@$imap->deleteMailbox($this->name);
	
    /* Delete the folder including sub-folders from the folder cache. */
		parent::delete();
	
    /* Delete the folder in the list of folders */
    unset($account->folders['main'][$this->name]);
 
    /* Delete groupware virtual folders too - FIXME move to ImapFolder->delete*/
    $oGWFolder = $this->getGWFolder($this->name); 

    if(!($oGWFolder===false))
    {
        if ($oGWFolder->type=='M') $oGWFolder->delete();
  	}

  	// Delete parent IMAP virtual folders
    if ($oFolderParent)
    {     
          //Are there any GW parent folders with the same name as parent folder
          //(is the parent folder virtual)

          $oParentGWFolder = $this->getGWFolder($oFolderParent->name);

          if (!($oParentGWFolder === false))
                $bDeleteFlag = true;
          else  $bDeleteFlag = false;
          
          
          //Get Childern of the parent IMAP folder
          $oChildern = $oFolderParent->getChildern($account);   

          //If no IMAP childern and Delete flag(virtual IMAP folder) 
          if ($oChildern===false && $deleteFlag)
          {
                
                $oGWFolder = $this->getGWFolder($oFolderParent->name);

                if ($oGWFolder->type!='M') $oFolderParent->delete();

          }
    }


	}
	
	/**
	  @brief Rename the folder.
	  @param $newName New name of the folder.
	 */
	public function rename($newName)
	{
	  if (DEBUG) echo "IMAPFolder->rename".CRLF;
		$account = $this->account;

		$imap = IMAP::instance($account);
    
		/* Rename the folder itself on the IMAP server. */
		$imap->renameMailbox($this->name, $newName);

		/* Rename the folder including sub-folders in the folder cache. */
		parent::rename($newName);
	}


	/**
	  @brief Delete multiple items
	 */
	public function deleteItems($oItems = false)
	{
		$account = $this->account;
		
    $imap = IMAP::instance($account);
						 
    $imap->deleteItems($this,$oItems);
    		
		parent::deleteItems($oItems);
	}
	
	/**
	  @brief Move items from the folder to another one.
	  @param $folder Destination folder object
	 */
	public function moveItems($oFolder,$oItems = false)
	{
		$account = $this->account;

		$imap = IMAP::instance($account);

		$imap->moveItems($this,$oFolder,$oItems);
		
		/* Move all items from the folder in the folder cache. */
		parent::moveItems($oFolder,$oItems);
	}
	/**
	  @brief Copy items from the folder to another one.
	  @param $folder Destination folder object
	 */	
	public function copyItems($oFolder,$oItems = false)
	{
		$account = $this->account;

		$imap = IMAP::instance($account);

		$imap->moveItems($this,$oFolder,$oItems,true);
		
		/* Move all items from the folder in the folder cache. */
		parent::copyItems($oFolder,$oItems);
	}	
	/*!
	  @brief Convert rights from IMAP format.
	  @param $rights The rights in IMAP format.
	  @return The converted rights.
	 */
	public static function decodeRights($imapRights)
	{
		$rights = 0;
		foreach (str_split($imapRights) as $right) {
			switch ($right) {
			case 'a': $rights |= Folder::RIGHT_ADMIN; break;
			case 'c': $rights |= Folder::RIGHT_CREATE; break;
			case 'd': $rights |= Folder::RIGHT_DELETE; break;
			case 'i': $rights |= Folder::RIGHT_INSERT; break;
			case 'l': $rights |= Folder::RIGHT_LOOKUP; break;
			case 'r': $rights |= Folder::RIGHT_READ; break;
			case 's': $rights |= Folder::RIGHT_SEEN; break;
			case 'w': $rights |= Folder::RIGHT_WRITE; break;
			}
		}

		return $rights;
	}

	/**
	  @brief Get the list of folder items.
	  @param $filter SQL WHERE contition.
	  @param $orderBy SQL ORDER BY condition.
	  @param $offset SQL OFFSET statement.
	  @param $limit SQL LIMIT statement.
	  @return The list as an array of IMAPItem objects.
	 */
	public function getItems($aFilterTag)
	{
		$filter = $aFilterTag['sql'];
		$orderBy = $aFilterTag['orderby'];
		$offset = $aFilterTag['offset'];
		$limit = $aFilterTag['limit'];
		$fields = $aFilterTag['tag'];

		// always get RID attribute of message
		if ($fields && !strstr($fields, 'rid'))
			$fields .= ',rid';

		return parent::getItems('IMAPItem', $filter, $orderBy, $offset, $limit,$fields);
	}

	/**
	  @brief Get an item.
	  @param $itemID ID of the item.
	  @return IMAPItem object.
	 */
	public function getItem($itemID, $cache = array())
	{

	  if (!$cache['cache']) $cache = false;
		return parent::getItem('IMAPItem', $itemID, $cache);
	}

	/**
	  @brief Create a new item.
	  @param $message The message of the item.
	 */
	public function createItem($message)
	{
		return IMAPItem::create($this, $message);
	}
	
	public function markItems($flag)
	{
    parent::markItems($flag);
		$account = $this->account;

    //Create instances of imap and cache classes
		$imap = IMAP::instance($account);    
						 
	  $imap->setFolderFlags($this->name,$flag);
  }
  
  public function unmarkItems($flag)
  {
    parent::unmarkItems($flag);   
		$account = $this->account;

    //Create instances of imap and cache classes
		$imap = IMAP::instance($account);    
						 
	  $imap->unsetFolderFlags($this->name,$flag);
  }

	/**
	  @brief Synchronize the item cache with the IMAP server.
	 */
	public function sync()
	{

		$account = $this->account;

    //Create instances of imap and cache classes
		$imap = IMAP::instance($account);
    

    $aItemsID = $imap->getItemsID($this->name);

    //Create temporarily unavailable
    file_put_contents($_SESSION['USERDIR'].SETTINGS_FOLDER.UNAVAILABLE_FILE,1);
    $oldExecutionTime = ini_alter('max_execution_time',MAX_EXECUTION_TIME);

    parent::sync($this->itemClassName,$aItemsID,$this);

    //Delete temporarily unavailable 
    @unlink($_SESSION['USERDIR'].SETTINGS_FOLDER.UNAVAILABLE_FILE);
    ini_set('max_execution_time',$oldExecutionTime);
	}
}

?>

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webmail/server/inc/imap/folder.php <==
<?php

require_once('inc/cache/folder.php');
require_once('inc/imap/imap.php');

/**
  @class IMAPFolder
  @brief Class representing an IMAP folder.
  @author Martin Zlomek
  @date 09-02-2006
 */
class IMAPFolder extends CacheFolder {

	//Type of the folder == M for mail items 
	public $type;  
	
	/**
	  @brief Create a new IMAP folder object.
	  @param $account Parent account object of the folder.
	  @param $folderID ID of the folder.
	  @param $name Name of the folder.
	  @param $rights Rights to the folder.
	 */	 
	public function __construct(&$account, $folderID, $name, $rights, $sync)
	{
		parent::__construct($account, $folderID, $name, $rights, $sync);
		$this->type = 'M';
		$this->gw = false;
		$this->itemClassName = 'IMAPItem';
	}

	/**
	  @brief Create a new folder.
	  @param $account Parent account object of the folder.
	  @param $name Name of the folder.
	  @return IMAPFolder object.
	 */
	public static function create($account, $name)
	{
    /* Create the folder on the IMAP server. */
		$imap = IMAP::instance($account);
		$rights = self::decodeRights($imap->createMailbox($name, $account->accountID));

		/* Create the folder in the folder cache. */
		$folderID = parent::create($account, $name, $rights);

		return new IMAPFolder($account, $folderID, $name, $rights, false);
	}


	/**
	  @brief Delete the folder.
	 */
	public function delete()
	{
		$account = $this->account;
		
		
	
		//Get parent folder
    $oFolderParent = $this->getParent($account);

  	$imap = IMAP::instance($account);

    $folders = krsort($account->folders["main"]);

		/* Delete all sub-folders from the IMAP server. */
		foreach ($account->folders["main"] as $folder) {
			if($_SESSION['DEBUG'])log_buffer("IMAPFolder->delete() Check for delete : ".$folder->name.'-'.$this->name.'/'."\r",'DEBUG');
			
      if (strpos($folder->name, $this->name . '/') !== 0)
				continue;
				
			if($_SESSION['DEBUG'])log_buffer("IMAPFolder->delete() ".$folder->name.'-'.$this->name.'/'."\r",'DEBUG');
			@$imap->deleteMailbox($folder->name);

			unset($account->folders['main'][$folder->name]);
		}   

		/* Delete the folder itself from the IMAP server. */
		@$imap->deleteMailbox($this->name);
	
    /* Delete the folder including sub-folders from the folder cache. */
		parent::delete();
	
    /* Delete the folder in the list of folders */
    unset($account->folders['main'][$this->name]);
 
    /* Delete groupware virtual folders too*/
    $oGWFolder = $this->getGWFolder($this->name); 

    if(!($oGWFolder===false))
    {
        if ($oGWFolder->type=='M') $oGWFolder->delete();
  	}

  	// Delete parent IMAP virtual folders
    if ($oFolderParent)
    {     
          //Are there any GW parent folders with the same name as parent folder
          //(is the parent folder virtual)

          $oParentGWFolder = $this->getGWFolder($oFolderParent->name);

          if (!($oParentGWFolder === false))
                $bDeleteFlag = true;
          else  $bDeleteFlag = false;
          
          
          //Get Childern of the parent IMAP folder
          $oChildern = $oFolderParent->getChildern($account);   

          //If no IMAP childern and Delete flag(virtual IMAP folder) 
          if ($oChildern===false && $deleteFlag)
          {
                
                $oGWFolder = $this->getGWFolder($oFolderParent->name);

                if ($oGWFolder->type!='M') $oFolderParent->delete();

          }
    }


	}
	
	/**
	  @brief Rename the folder.
	  @param $newName New name of the folder.
	 */
	public function rename($newName)
	{
	  if($_SESSION['DEBUG'])log_buffer("IMAPFolder->rename",'DEBUG');
		$account = $this->account;


		$imap = IMAP::instance($account);
    $type = 'M';
    $parent = substr($newName,0,strrpos($newName,$imap->delimiter));
    
    //Check for parent folder
    if($parent){
      try{
        $oFolder = $account->getFolder($parent,$type);
      }catch(Exc $e){
        
        $imap->createMailbox($parent);   
      }
    }
    
		/* Rename the folder itself on the IMAP server. */
		$imap->renameMailbox($this->name, $newName);

		/* Rename the folder including sub-folders in the folder cache. */
		parent::rename($newName);
	}


	/**
	  @brief Delete multiple items
	 */
	public function deleteItems($oItems = false)
	{
		$account = $this->account;
    	$imap = IMAP::instance($account);
		if($oItems === false){
			$sequence = $this->getItemsSequence();
			$imap->deleteItems($this,$oItems,$sequence);
		}else{	
			$imap->deleteItems($this,$oItems);
		}				 
		parent::deleteItems($oItems);
	}
	
	/**
	  @brief Move items from the folder to another one.
	  @param $folder Destination folder object
	 */
	public function moveItems($oFolder,$oItems = false)
	{
		$account = $this->account;
		$imap = IMAP::instance($account);	
		if($oItems === false){
			$sequence = $this->getItemsSequence();
			$aMovedID = $imap->moveItems($this,$oFolder,$oItems,false,$sequence);
		}else{	
			$aMovedID = $imap->moveItems($this,$oFolder,$oItems);
		}
		/* Move all items from the folder in the folder cache. */
		parent::moveItems($oFolder,$oItems,$aMovedID);
	}
	
	/**
	  @brief Copy items from the folder to another one.
	  @param $folder Destination folder object
	 */	
	public function copyItems($oFolder,$oItems = false)
	{
		$account = $this->account;
		$imap = IMAP::instance($account);
		if($oItems === false){
			$sequence = $this->getItemsSequence();
			$aCopiedID = $imap->moveItems($this,$oFolder,$oItems,true,$sequence);
		}else{	
			$aCopiedID = $imap->moveItems($this,$oFolder,$oItems,true);
		}
		
		/* Copy all items from the folder in the folder cache. */
		parent::copyItems($oFolder,$oItems,$aCopiedID);
	}	
	/*!
	  @brief Convert rights from IMAP format.
	  @param $rights The rights in IMAP format.
	  @return The converted rights.
	 */
	public static function decodeRights($imapRights)
	{
		$rights = 0;

		foreach (str_split($imapRights) as $right) {
			switch ($right) {
			case 'a': $rights |= Folder::RIGHT_ADMIN; break;
			case 'c': $rights |= Folder::RIGHT_CREATE; break;
			case 'i': $rights |= Folder::RIGHT_INSERT; break;
			case 'l': $rights |= Folder::RIGHT_LOOKUP; break;
			case 'r': $rights |= Folder::RIGHT_READ; break;
			case 'k': $rights |= Folder::RIGHT_MAILBOX_WRITE; break;
			case 'd':  $rights |= Folder::RIGHT_D; break;
			case 'c':  $rights |= Folder::RIGHT_C; break;
      		case 't': $rights |= Folder::RIGHT_DELETE; break;
      		case 'x': $rights |= Folder::RIGHT_MAILBOX_DELETE; break;
			case 's': $rights |= Folder::RIGHT_SEEN; break;
			case 'w': $rights |= Folder::RIGHT_WRITE; break;
			case 'e': $rights |= Folder::RIGHT_EXPUNGE; break;
			}
		}
		return $rights;
	}

	/**
	  @brief Get the list of folder items.
	  @param $filter SQL WHERE contition.
	  @param $orderBy SQL ORDER BY condition.
	  @param $offset SQL OFFSET statement.
	  @param $limit SQL LIMIT statement.
	  @return The list as an array of IMAPItem objects.
	 */
	public function getItems($aFilterTag = array('tag'=>'*'))
	{
		$filter = $aFilterTag['sql'];
		$orderBy = $aFilterTag['orderby'];
		$offset = $aFilterTag['offset'];
		$limit = $aFilterTag['limit'];
		$fields = $aFilterTag['tag'];
    $search = $aFilterTag['fulltext'];
    
		// always get RID attribute of message
		if ($fields && !strstr($fields, 'rid'))
			$fields .= ',rid';

		return parent::getItems('IMAPItem', $filter, $orderBy, $offset, $limit,$fields,$search);
	}

	/**
	  @brief Get an item.
	  @param $itemID ID of the item.
	  @return IMAPItem object.
	 */
	public function getItem($itemID, $cache = array())
	{

	  if (!$cache['cache']) $cache = false;
		return parent::getItem('IMAPItem', $itemID, $cache);
	}

	/**
	  @brief Create a new item.
	  @param $message The message of the item.
	 */
	public function createItem($message = false,$file = false,$bNoCache = false)
	{
		return IMAPItem::create($this, $message, $file,$bNoCache);
	}
	
	public function markItems($flag)
	{
    	parent::markItems($flag);
		$account = $this->account;

    	//Create instances of imap and cache classes
		$imap = IMAP::instance($account);    
		$sequence = $this->getItemsSequence();
		$imap->setFlags( $this->name, $sequence, IMAPItem::encodeFlags( $flag ) );
  }
  
  public function unmarkItems($flag)
  {
    	parent::unmarkItems($flag);   
		$account = $this->account;

    	//Create instances of imap and cache classes
		$imap = IMAP::instance($account);    
		$sequence = $this->getItemsSequence();
		$imap->setFlags( $this->name, $sequence, IMAPItem::encodeFlags( $flag ) );
  }

	/**
	  @brief Synchronize the item cache with the IMAP server.
	 */
	public function sync()
	{
		try{
			$account = $this->account;
			
			//Create instances of imap and cache classes
			$imap = IMAP::instance($account);
			$aItemsID = $imap->getItemsID($this->name);
			//Create temporarily unavailable
			@file_put_contents($_SESSION['USERDIR'].SETTINGS_FOLDER.UNAVAILABLE_FILE,1);
			$oldExecutionTime = ini_alter('max_execution_time',MAX_EXECUTION_TIME);
			
			parent::sync($this->itemClassName,$aItemsID,$this);
			
			//Delete temporarily unavailable 
			@unlink($_SESSION['USERDIR'].SETTINGS_FOLDER.UNAVAILABLE_FILE);
			ini_set('max_execution_time',$oldExecutionTime);
		}catch(Exc $e){
			
		}catch(Exception $e){
			
		}
	}
	
  public function search($sPhrase,$aFilterTag){

    if($_SESSION['FULLTEXT'])
      return parent::search($sPhrase,$aFilterTag);
    else{
      $imap = IMAP::instance($this->account);  
      $result = $imap->search($this->name,$sPhrase);
    }
    
    $aFilterTag = $this->updateSearchFilter($result,$aFilterTag);
    $result = $this->getItems($aFilterTag);
    
    return $result;
  }
  
  /**
   * IMAPFolder::getItemsSequence()
   * 
   * Returns sequence of the lowest and the highest id
   * of the messages in item cache
   * 
   * @return $seq 'lower:upper'
   */
  private function getItemsSequence()
  {
  	$cache = Cache::instance( $_SESSION['user'] );
	$seq = $cache->query(
	'SELECT min(rid) as lower,max(rid) as upper FROM item'.
	' WHERE folder_id = '.$this->folderID
	);
	$result = $seq->fetch();
	return $result['lower'].':'.$result['upper'];
  }
  
}

?>
